/**********************************************************************
 * Copyright (c) 1995 MPEG/audio software simulation group
 * All Rights Reserved
 * $Id: common.c,v 1.3 1995/04/06 05:01:42 rowlands Exp $
 *
 * MPEG/audio coding/decoding software, work in progress
 *   NOT for public distribution until verified and approved by the
 *   MPEG/audio committee.
 *
 * $Log: common.c,v $
 * Revision 1.3  1995/04/06  05:01:42  rowlands
 * Ran decoder through protoize to convert to ANSI C.
 *
 * Revision 1.2  1995/04/06  04:56:00  rowlands
 * Added header, RCS info and cleaned up prototypes in initial revision.
 *
 **********************************************************************/

/**********************************************************************
 * VERSION 2.5                                                        *
 *   changes made since last update:                                  *
 *   date   programmers                comment                        *
 * 2/25/91  Douglas Wong        start of version 1.0 records          *
 * 3/06/91  Douglas Wong        rename setup.h to dedef.h             *
 *                              removed extraneous variables          *
 *                              removed window_samples (now part of   *
 *                              filter_samples)                       *
 * 3/07/91  Davis Pan           changed output file to "codmusic"     *
 * 5/10/91  Vish (PRISM)        Ported to Macintosh and Unix.         *
 *                              Incorporated new "out_fifo()" which   *
 *                              writes out last incomplete buffer.    *
 *                              Incorporated all AIFF routines which  *
 *                              are also compatible with SUN.         *
 *                              Incorporated user interface for       *
 *                              specifying sound file names.          *
 *                              Also incorporated user interface for  *
 *                              writing AIFF compatible sound files.  *
 * 27jun91  dpwe (Aware)        Added musicout and &sample_frames as  *
 *                              args to out_fifo (were glob refs).    *
 *                              Used new 'frame_params' struct.       *
 *                              Clean,simplify, track clipped output  *
 *                              and total bits/frame received.        *
 * 7/10/91  Earle Jennings      changed to floats to FLOAT            *
 *10/ 1/91  S.I. Sudharsanan,   Ported to IBM AIX platform.           *
 *          Don H. Lee,                                               *
 *          Peter W. Farrett                                          *
 *10/ 3/91  Don H. Lee          implemented CRC-16 error protection   *
 *                              newly introduced functions are        *
 *                              buffer_CRC and recover_CRC_error      *
 *                              Additions and revisions are marked    *
 *                              with "dhl" for clarity                *
 * 2/11/92  W. Joseph Carter    Ported new code to Macintosh.  Most   *
 *                              important fixes involved changing     *
 *                              16-bit ints to long or unsigned in    *
 *                              bit alloc routines for quant of 65535 *
 *                              and passing proper function args.     *
 *                              Removed "Other Joint Stereo" option   *
 *                              and made bitrate be total channel     *
 *                              bitrate, irrespective of the mode.    *
 *                              Fixed many small bugs & reorganized.  *
 **********************************************************************
 *                                                                    *
 *                                                                    *
 *  MPEG/audio Phase 2 coding/decoding multichannel                   *
 *                                                                    *
 *  Version 1.0                                                       *
 *                                                                    *
 *  7/27/93        Susanne Ritscher,  IRT Munich                      *
 *                                                                    *
 *                  thanks to                                         *
 *                  Ralf Schwalbe,    Telekom FTZ Berlin              *
 *                  Heiko Purnhagen,  Uni Hannover                    *
 *                                                                    *
 *  Version 2.0                                                       *
 *                                                                    *
 *  8/27/93        Susanne Ritscher, IRT Munich                       *
 *                 Channel-Switching is working                       *
 *                                                                    *
 *  Version 2.1                                                       *
 *                                                                    *
 *  9/1/93         Susanne Ritscher,  IRT Munich                      *
 *                 all channels normalized                            *
 *                                                                    *
 *  Version 3.0                                                       *
 *                                                                    *
 *  06/16/94       Ralf Schwalbe, Telekom FTZ Berlin                  *
 *                 all sources and variables adapted due to MPEG-2 -  *
 *                 DIS from March 1994                                *
 *                  - dematrix and denormalize procedure              *
 *                  - new tc - allocation (0-7)                       *
 *                  - some new structures and variables as a basis    *
 *                    for further decoding modes                      *
 **********************************************************************
 *								      *
 *  Version 1.0 Shareware                                             *
 *                                                                    *
 *  07/12/94       Ralf Schwalbe,  Telekom FTZ Berlin                 *
 *                 Tel: +49 30 6708 2406                              *
 *                 Fax: +49 30 6774 539                               *
 *								      *
 *  04/11/94	   Ralf Schwalbe. Telekom FTZ Berlin                  *
 *                  - decoding extension bitstream                    *
 *                  - some new subroutines, globale variables and     *
 *                    structures (important to handle the ext. bitst.)*
 *                  - changed all functions to ANSI-C funktion header *
 *		    - corrected some bugs to decode bitstreams > 512kB*
 *								      *
 *  Version 1.1 Shareware                                             *
 *                                                                    *
 *  07/12/94       Ralf Schwalbe,  Telekom FTZ Berlin                 *
 *                 Tel: +49 30 6708 2406                              *
 *                 Fax: +49 30 6774 539                               *
 *								      *
 **********************************************************************/

/***********************************************************************
*
*  Global Include Files
*
***********************************************************************/

#include        "common.h"
#include        "decoder.h" /* 23/03/1995 JMZ Test_bit_stream */

#ifdef  MACINTOSH

#include        <SANE.h>
#include        <pascal.h>

#endif

#include <string.h>
#include <ctype.h>

/***********************************************************************
*
*  Global Variable Definitions
*
***********************************************************************/
extern int bits_in_frame;
char *mode_names[4] = { "stereo", "j-stereo", "dual-ch", "single-ch" };
char *layer_names[3] = { "I", "II", "III" };

double  s_freq[4] = {44.1, 48, 32, 0};

int	  bitrate[3][15] = {
			 {1000,32,64,96,128,160,192,224,256,288,320,352,384,416,448},
			 {1000,32,48,56,64,80,96,112,128,160,192,224,256,320,384},
			 {1000,32,40,48,56,64,80,96,112,128,160,192,224,256,320}
        };

double /*far*/ multiple[64] = {
2.00000000000000, 1.58740105196820, 1.25992104989487,
1.00000000000000, 0.79370052598410, 0.62996052494744, 0.50000000000000,
0.39685026299205, 0.31498026247372, 0.25000000000000, 0.19842513149602,
0.15749013123686, 0.12500000000000, 0.09921256574801, 0.07874506561843,
0.06250000000000, 0.04960628287401, 0.03937253280921, 0.03125000000000,
0.02480314143700, 0.01968626640461, 0.01562500000000, 0.01240157071850,
0.00984313320230, 0.00781250000000, 0.00620078535925, 0.00492156660115,
0.00390625000000, 0.00310039267963, 0.00246078330058, 0.00195312500000,
0.00155019633981, 0.00123039165029, 0.00097656250000, 0.00077509816991,
0.00061519582514, 0.00048828125000, 0.00038754908495, 0.00030759791257,
0.00024414062500, 0.00019377454248, 0.00015379895629, 0.00012207031250,
0.00009688727124, 0.00007689947814, 0.00006103515625, 0.00004844363562,
0.00003844973907, 0.00003051757813, 0.00002422181781, 0.00001922486954,
0.00001525878906, 0.00001211090890, 0.00000961243477, 0.00000762939453,
0.00000605545445, 0.00000480621738, 0.00000381469727, 0.00000302772723,
0.00000240310869, 0.00000190734863, 0.00000151386361, 0.00000120155435,
1E-20
};

int sb_groups[12] = { 0, 1, 2, 3, 4, 5, 6, 7, 9, 11, 15, 26};

/*int transmission_channel[8][5] = {{0, 1, 2, 3, 4},
				    {0, 1, 5, 3, 4},
				    {0, 1, 6, 3, 4},
				    {0, 1, 2, 5, 4},
				    {0, 1, 2, 3, 6},
				    {0, 1, 2, 5, 6},
				    {0, 1, 6, 5, 4},
				    {0, 1, 5, 3, 6}};
*/

/***********************************************************************
*
*  Global Function Definitions
*
***********************************************************************/

/* The system uses a variety of data files.  By opening them via this
   function, we can accommodate various locations. */

FILE *OpenTableFile(char *name)
{
char fulname[80];
char *envdir;
FILE *f;

	 fulname[0] = '\0';

#ifdef TABLES_PATH
	   strcpy(fulname, TABLES_PATH);   /* default relative path for tables */
#endif /* TABLES_PATH */          /* (includes terminal path seperator */

#ifdef UNIX                       /* envir. variables for UNIX only */
	   {
		char *getenv(const char *);

        envdir = getenv(MPEGTABENV);   /* check for environment */
        if(envdir != NULL)
            strcpy(fulname, envdir);
        strcat(fulname, PATH_SEPARATOR);  /* add a "/" on the end */
      }
#endif /* UNIX */

	strcat(fulname, name);
    if( (f=fopen(fulname,"r"))==NULL ) {
        fprintf(stderr,"OpenTable: could not find %s\n", fulname);

#ifdef UNIX
          if(envdir != NULL)
			fprintf(stderr,"Check %s directory '%s'\n",MPEGTABENV, envdir);
          else
            fprintf(stderr,"Check local directory './%s' or setenv %s\n",
                    TABLES_PATH, MPEGTABENV);
#else /* not unix : no environment variables */

#ifdef TABLES_PATH
            fprintf(stderr,"Check local directory './%s'\n",TABLES_PATH);
#endif /* TABLES_PATH */

#endif /* UNIX */

    }
    return f;
}

/***********************************************************************
/*
/* Read one of the data files ("alloc_*") specifying the bit allocation/
/* quatization parameters for each subband in layer II encoding
/*
/**********************************************************************/
/* read in table, return # subbands */
int read_bit_alloc(int table, al_table *alloc)  

{
        long a, b, c, d, i, j;
		FILE *fp;
        char name[16], t[80];
        int sblim;

        strcpy(name, "alloc_0");

		switch (table) {
                case 0 : name[6] = '0';         break;
                case 1 : name[6] = '1';         break;
                case 2 : name[6] = '2';         break;
                case 3 : name[6] = '3';         break;
                default : name[6] = '0';
		}

        if (!(fp = OpenTableFile(name))) {
                printf("Please check bit allocation table %s\n", name);
                exit(0);
        }

        printf("using bit allocation table %s\n", name);

        fgets(t, 80, fp);
        sscanf(t, "%d\n", &sblim);
        while (!feof(fp)) {
				fgets(t, 80, fp);
                sscanf(t, "%ld %ld %ld %ld %ld %ld\n", &i, &j, &a, &b, &c, &d);
						(*alloc)[i][j].steps = a;
                        (*alloc)[i][j].bits  = b;
                        (*alloc)[i][j].group = c;
                        (*alloc)[i][j].quant = d;
        }
        fclose(fp);
		return sblim;
}

/***********************************************************************
/*
/* Using the decoded info the appropriate possible quantization per
/* subband table is loaded
/*
/**********************************************************************/
/* choose table, load if necess, return # sb's */
int pick_table(frame_params *fr_ps)   
{
        int table, lay, ws, bsp, br_per_ch, sfrq;
		int sblim = fr_ps->sblimit;     /* return current value if no load */

		lay = fr_ps->header->lay - 1;
        bsp = fr_ps->header->bitrate_index;
		  br_per_ch = bitrate[lay][bsp] / (fr_ps->stereo + fr_ps->mc_channel);
        ws = fr_ps->header->sampling_frequency;
		sfrq = s_freq[ws];
        /* decision rules refer to per-channel bitrates (kbits/sec/chan) */
        if ((sfrq == 48 && br_per_ch >= 56) ||
            (br_per_ch >= 56 && br_per_ch <= 80)) table = 0;
        else if (sfrq != 48 && br_per_ch >= 96) table = 1;
        else if (sfrq != 32 && br_per_ch <= 48) table = 2;
        else table = 3;
        if (fr_ps->tab_num != table) {
		   if (fr_ps->tab_num >= 0)
              mem_free((void **)&(fr_ps->alloc));
           fr_ps->alloc = (al_table /*far*/ *) mem_alloc(sizeof(al_table),
                                                         "alloc");
           sblim = read_bit_alloc(fr_ps->tab_num = table, fr_ps->alloc);
        }
		return sblim;
}


int js_bound(int lay, int m_ext)
{
static int jsb_table[3][4] =  { { 4, 8, 12, 16 }, { 4, 8, 12, 16},
								{ 0, 4, 8, 16} };  /* lay+m_e -> jsbound */

    if(lay<1 || lay >3 || m_ext<0 || m_ext>3) {
        fprintf(stderr, "js_bound bad layer/modext (%d/%d)\n", lay, m_ext);
        exit(0);
    }
    return(jsb_table[lay-1][m_ext]);
}

/* other bounderies for multichannel*/
int js_bound1(int lay, int m_ext)
{
static int jsb_table[3][8] =	{ { 0, 0, 0, 0, 0, 0, 0, 0 }, { 4, 5, 6, 7, 8, 10, 12, 16},
										  { 0, 0, 0, 0, 0, 0, 0, 0 } };	/* lay+m_e -> jsbound */

	if(lay<1 || lay >3 || m_ext<0 || m_ext>3) {
		fprintf(stderr, "js_bound bad layer/modext (%d/%d)\n", lay, m_ext);
		exit(0);
	}
	return(jsb_table[lay-1][m_ext]);
}

/* interpret data in hdr str to fields in fr_ps */
void hdr_to_frps(frame_params *fr_ps)
{
layer *hdr = fr_ps->header;     /* (or pass in as arg?) */

/* 09/03/1995 JMZ Multilingual */
int		surround = hdr->surround;
int		center = hdr->center;
int 	mc_channel = 0;
/* 09/03/1995 JMZ Multilingual */

	fr_ps->actual_mode = hdr->mode;
	if(hdr->mode != MPG_MD_NONE)
		 fr_ps->stereo = (hdr->mode == MPG_MD_MONO) ? 1 : 2;
	else fr_ps->stereo = 0;

/* 09/03/1995 JMZ Multilingual */
	/*fr_ps->mc_channel = 3; */ /* R.S. only for 3 mc-channels */
	switch (surround)
		{
		case 0:	mc_channel = 0; break;
		case 1:	mc_channel = 1; break;
		case 2:	mc_channel = 2; break;
		case 3:	mc_channel = 0; break;
		}
	switch (center)
		{
		case 0:	break;
		case 1:	mc_channel ++; break;
		case 2:	break;
		case 3:	mc_channel ++; break;
		}
	fr_ps->mc_channel = mc_channel; 
/* 09/03/1995 JMZ Multilingual */

	if (hdr->lay == 2)          fr_ps->sblimit = pick_table(fr_ps);
	else                        fr_ps->sblimit = SBLIMIT;
	if(hdr->mode == MPG_MD_JOINT_STEREO)
		  fr_ps->jsbound = js_bound(hdr->lay, hdr->mode_ext);
	else
		  fr_ps->jsbound = fr_ps->sblimit;

}

void WriteHdr(frame_params *fr_ps, FILE *s)
{
layer *info = fr_ps->header;

   fprintf(s, "HDR:  s=FFF, id=%X, l=%X, ep=%X, br=%X, sf=%X, pd=%X, ",
		   info->version, info->lay, !info->error_protection,
		   info->bitrate_index, info->sampling_frequency, info->padding);
   fprintf(s, "pr=%X, m=%X, js=%X, c=%X, o=%X, e=%X\n",
		   info->extension, info->mode, info->mode_ext,
		   info->copyright, info->original, info->emphasis);
   fprintf(s, "layer=%s, MPEG1-bitrate=%d, sfrq=%.1f, mode=%s, ",
		   layer_names[info->lay-1], bitrate[info->lay-1][info->bitrate_index],
		   s_freq[info->sampling_frequency], mode_names[info->mode]);
   fprintf(s, "sblim=%d, jsbd=%d, MPEG1-ch=%d\n",
		   fr_ps->sblimit, fr_ps->jsbound, fr_ps->stereo);
   fflush(s);
   
}

void WriteBitAlloc(unsigned int bit_alloc[2][SBLIMIT], frame_params *f_p, FILE *s)
{
int i,j;
int st = f_p->stereo;
int sbl = f_p->sblimit;
int jsb = f_p->jsbound;

	fprintf(s, "BITA ");
	for(i=0; i<sbl; ++i) {
		if(i == jsb) fprintf(s,"-");
		for(j=0; j<st; ++j)
            fprintf(s, "%1x", bit_alloc[j][i]);
	}
	fprintf(s, "\n");   fflush(s);
}

void WriteScale(unsigned int bit_alloc[2][SBLIMIT],
		unsigned int scfsi[2][SBLIMIT],	
		unsigned int scalar[2][3][SBLIMIT],
		frame_params *fr_ps, FILE *s)
{
int stereo  = fr_ps->stereo;
int sblimit = fr_ps->sblimit;
/*int jsbound = fr_ps->jsbound;*/
int lay     = fr_ps->header->lay;
int i,j,k;

		if(lay == 2) {
			fprintf(s, "SCFSI ");
			for (i=0;i<sblimit;i++) for (k=0;k<stereo;k++)
				if (bit_alloc[k][i])  fprintf(s,"%d",scfsi[k][i]);
			fprintf(s, "\nSCFs ");
			for (k=0;k<stereo;k++) {
				for (i=0;i<sblimit;i++)
                    if (bit_alloc[k][i])
                        switch (scfsi[k][i]) {
                          case 0: for (j=0;j<3;j++)
                                  fprintf(s,"%2d%c",scalar[k][j][i],
                                          (j==2)?';':'-');
                                  break;
						  case 1:
                          case 3: fprintf(s,"%2d-",scalar[k][0][i]);
                                  fprintf(s,"%2d;",scalar[k][2][i]);
                                  break;
                          case 2: fprintf(s,"%2d;",scalar[k][0][i]);
						}
                fprintf(s, "\n");
            }
        }
        else{   /* lay == 1 */
			fprintf(s, "SCFs ");
            for (i=0;i<sblimit;i++) for (k=0;k<stereo;k++)
				if (bit_alloc[k][i])  fprintf(s,"%2d;",scalar[k][0][i]);
			fprintf(s, "\n");
        }
}

void WriteSamples(int ch, unsigned int sample[SBLIMIT], 
		  unsigned int bit_alloc[SBLIMIT], 
		  frame_params *fr_ps, FILE *s)
{
int i,j,k;
int stereo = fr_ps->stereo;
int sblimit = fr_ps->sblimit;

        fprintf(s, "SMPL ");
        for (i=0;i<sblimit;i++)
				if ( bit_alloc[i] != 0)
                    fprintf(s, "%d:", sample[i]);
        if(ch==(stereo-1) )     fprintf(s, "\n");
		else                    fprintf(s, "\t");
}

int NumericQ(char *s) /* see if a string looked like a numeric argument */
{
/*int     ans = 0;  */ /*21.9.93 R.S. not used yet */
char    c;

    while( (c = *s++)!='\0' && isspace((int)c)) /* strip leading ws */
        ;
    if( c == '+' || c == '-' )
		c = *s++;               /* perhaps skip leading + or - */
    return isdigit((int)c);
}

int BitrateIndex(int layr, int bRate)   /* convert bitrate in kbps to index */
{
int     index = 0;
int     found = 0;

    while(!found && index<15)   {
        if(bitrate[layr-1][index] == bRate)
            found = 1;
        else
			++index;
    }
    if(found)
        return(index);
    else {
		fprintf(stderr, "BitrateIndex: %d (layer %d) is not a legal bitrate\n",
                bRate, layr);
        return(-1);     /* Error! */
    }
}

int SmpFrqIndex(long sRate)  /* convert samp frq in Hz to index ,legal rates 32000, 44100, 48000 */
{
	if(sRate == 44100L)
		return(0);
	else if(sRate == 48000L)
		return(1);
    else if(sRate == 32000L)
        return(2);
    else {
        fprintf(stderr, "SmpFrqIndex: %ld is not a legal sample rate\n", sRate);
        return(-1);     /* Error! */
    }
}

/*******************************************************************************
*
*  Allocate number of bytes of memory equal to "block".
*
*******************************************************************************/

void    *mem_alloc(unsigned long block, char *item)
{
void	  *ptr;

#ifdef  MACINTOSH
    ptr = NewPtr(block);
#endif

#ifdef  MSDOS
	 ptr = (void *)_fmalloc((unsigned int)block); /* far memory, 92-07-08 sr */
#endif

#if ! defined (MACINTOSH) && ! defined (MSDOS)
    ptr = (void *) malloc(block);
#endif
	 if (ptr != NULL){
#ifdef  MSDOS
	_fmemset(ptr, 0, (unsigned int)block); /* far memory, 92-07-08 sr */
#else
		memset(ptr, 0, block);
#endif
    }
    else{
        printf("Unable to allocate %s\n", item);
        exit(0);
    }

    return(ptr);

}

/****************************************************************************
*
*  Free memory pointed to by "*ptr_addr".
*
*****************************************************************************/

void    mem_free(void **ptr_addr)
{

    if (*ptr_addr != NULL){
#ifdef  MACINTOSH
        DisposPtr(*ptr_addr);
#else
        free(*ptr_addr);
#endif
        *ptr_addr = NULL;
    }

}

/*******************************************************************************
*
*  Check block of memory all equal to a single byte, else return FALSE
*
*******************************************************************************/

int memcheck(char *array, int test, int num)
            
                /* but only tested as a char (bottom 8 bits) */
        
{
 int i=0;

   while (array[i] == test && i<num) i++;
   if (i==num) return TRUE;
   else return FALSE;
}

/****************************************************************************
*
*  Routines to convert between the Apple SANE extended floating point format
*  and the IEEE double precision floating point format.  These routines are
*  called from within the Audio Interchange File Format (AIFF) routines.
*
*****************************************************************************/

/*
*** Apple's 80-bit SANE extended has the following format:

 1       15      1            63
+-+-------------+-+-----------------------------+
|s|       e     |i|            f                |
+-+-------------+-+-----------------------------+
  msb        lsb   msb                       lsb

The value v of the number is determined by these fields as follows:
If 0 <= e < 32767,              then v = (-1)^s * 2^(e-16383) * (i.f).
If e == 32767 and f == 0,       then v = (-1)^s * (infinity), regardless of i.
If e == 32767 and f != 0,       then v is a NaN, regardless of i.

*** IEEE Draft Standard 754 Double Precision has the following format:

MSB
+-+---------+-----------------------------+
|1| 11 Bits |           52 Bits           |
+-+---------+-----------------------------+
 ^     ^                ^
 |     |                |
 Sign  Exponent         Mantissa
*/

/*****************************************************************************
*
*  double_to_extended()
*
*  Purpose:     Convert from IEEE double precision format to SANE extended
*               format.
*
*  Passed:      Pointer to the double precision number and a pointer to what
*               will hold the Apple SANE extended format value.
*
*  Outputs:     The SANE extended format pointer will be filled with the
*               converted value.
*
*  Returned:    Nothing.
*
*****************************************************************************/


void    double_to_extended(double *pd, char ps[10])
{

#ifdef  MACINTOSH

        x96tox80(pd, (extended *) ps);

#else

/* fixed bus alignment error, HP 27-may-93 */

register unsigned long  top2bits;

register unsigned short *ps2;
register IEEE_DBL       *p_dbl;
register SANE_EXT       *p_ext;
SANE_EXT ext_align;
char *c_align;
int i;

   p_dbl = (IEEE_DBL *) pd;
   p_ext = &ext_align;
   top2bits = p_dbl->hi & 0xc0000000L;
   p_ext->l1 = ((p_dbl->hi >> 4) & 0x3ff0000L) | top2bits;
   p_ext->l1 |= ((p_dbl->hi >> 5) & 0x7fffL) | 0x8000L;
   p_ext->l2 = (p_dbl->hi << 27) & 0xf8000000L;
   p_ext->l2 |= ((p_dbl->lo >> 5) & 0x07ffffffL);
   ps2 = (unsigned short *) & (p_dbl->lo);
   ps2++;
   p_ext->s1 = (*ps2 << 11) & 0xf800;

   c_align = (char *) p_ext;
   for (i=0;i<10;i++)
	 ps[i] = c_align[i];

#endif

}


/*****************************************************************************
*
*  extended_to_double()
*
*  Purpose:     Convert from SANE extended format to IEEE double precision
*               format.
*
*  Passed:      Pointer to the Apple SANE extended format value and a pointer
*               to what will hold the the IEEE double precision number.
*
*  Outputs:     The IEEE double precision format pointer will be filled with
*               the converted value.
*
*  Returned:    Nothing.
*
*****************************************************************************/

void    extended_to_double(char ps[10], double *pd)
{

#ifdef  MACINTOSH

   x80tox96((extended *) ps, pd);

#else

/* fixed bus alignment error, HP 27-may-93 */

register unsigned long  top2bits;

register IEEE_DBL       *p_dbl;
register SANE_EXT       *p_ext;
SANE_EXT ext_align;
char *c_align;
int i;

   p_dbl = (IEEE_DBL *) pd;
   p_ext = &ext_align;

   c_align = (char *) p_ext;
   for (i=0;i<10;i++)
	 c_align[i] = ps[i];

   top2bits = p_ext->l1 & 0xc0000000L;
   p_dbl->hi = ((p_ext->l1 << 4) & 0x3ff00000L) | top2bits;
   p_dbl->hi |= (p_ext->l1 << 5) & 0xffff0L;
   p_dbl->hi |= (p_ext->l2 >> 27) & 0x1f;
   p_dbl->lo = (p_ext->l2 << 5) & 0xffffffe0L;
   p_dbl->lo |= (unsigned long) ((p_ext->s1 >> 11) & 0x1f);

#endif

}

/*****************************************************************************
*
*  Read Audio Interchange File Format (AIFF) headers.
*
*****************************************************************************/

int aiff_read_headers(FILE *file_ptr,
		      IFF_AIFF *aiff_ptr, 
		      int *byte_per_sample)
{

register char   i;
register long   seek_offset;

char		temp_sampleRate[10];
char		*dummy;
char		holder;
Chunk           FormChunk;
CommonChunk     CommChunk;
SoundDataChunk  SndDChunk;
identifier	ident;

   if (fseek(file_ptr, 0, SEEK_SET) != 0)
	  return(-1);

   if (fread(&FormChunk, sizeof(Chunk), 1, file_ptr) != 1)
	  return(-1);

#ifdef MSDOS

  holder = FormChunk.ckID[0];
  FormChunk.ckID[0] = FormChunk.ckID[3];
  FormChunk.ckID[3] = holder;
  holder = FormChunk.ckID[1];
  FormChunk.ckID[1] = FormChunk.ckID[2];
  FormChunk.ckID[2] = holder;

/* fixed bug in next line, HP 27-may-93 */
  holder = FormChunk.formType[0];
  FormChunk.formType[0] = FormChunk.formType[3];
  FormChunk.formType[3] = holder;
  holder = FormChunk.formType[1];
  FormChunk.formType[1] = FormChunk.formType[2];
  FormChunk.formType[2] = holder;

  FormChunk.ckSize = _lrotl(FormChunk.ckSize, 8);
#endif


/* fixed bug in next line, HP 27-may-93 */
	if (strncmp(FormChunk.ckID,IFF_ID_FORM,4) != 0 ||
		 strncmp(FormChunk.formType,IFF_ID_AIFF,4) != 0)
      return(-1); 	/* warning: different levels of indirection.7/8/92.sr*/

	if (fread(&ident,sizeof(identifier), 1, file_ptr) != 1)
		return(-1);

#ifdef MSDOS

	holder = ident.name[0];
	ident.name[0] = ident.name[3];
	ident.name[3] = holder;
	holder = ident.name[1];
	ident.name[1] = ident.name[2];
	ident.name[2] = holder;

	ident.ck_length = _lrotl(ident.ck_length, 8);

#endif

/* fixed bug in next line, HP 27-may-93 */
	while(strncmp(ident.name,IFF_ID_COMM,4) != 0)
	{
		dummy = calloc( ident.ck_length, sizeof(char));
/* changed "fread( &dummy," to "fread ( dummy,", HP 26-may-93 */
		if(fread( dummy, ident.ck_length, 1, file_ptr) != 1)
		  return(-1);
		free(dummy);
/* fixed bug in next line, HP 27-may-93 */
		if(fread( &ident, sizeof(identifier),1, file_ptr) != 1)
		  return(-1);

#ifdef MSDOS

	holder = ident.name[0];
	ident.name[0] = ident.name[3];
	ident.name[3] = holder;
	holder = ident.name[1];
	ident.name[1] = ident.name[2];
	ident.name[2] = holder;

	ident.ck_length = _lrotl(ident.ck_length, 8);

#endif
	}

		for( i = 0; i < 4; ++i)
		CommChunk.ckID[i] = ident.name[i];

		CommChunk.ckSize = ident.ck_length;

		if (fread(&CommChunk.numChannels, sizeof(short), 1, file_ptr) != 1)
			return(-1);

		if (fread(&CommChunk.numSampleFrames, sizeof(unsigned long), 1,
			 file_ptr) != 1)
			return(-1);

		if (fread(&CommChunk.sampleSize, sizeof(short), 1, file_ptr) != 1)
			return(-1);

		if (fread(CommChunk.sampleRate, sizeof(char[10]), 1, file_ptr) != 1)
			return(-1);

#ifdef MSDOS

  CommChunk.sampleSize = _rotl(CommChunk.sampleSize, 8);
  CommChunk.ckSize = _lrotl(CommChunk.ckSize, 8);
  CommChunk.numChannels = _rotl(CommChunk.numChannels, 8);
  CommChunk.numSampleFrames = _lrotl(CommChunk.numSampleFrames, 8);

#endif


  *byte_per_sample = ceil((double)CommChunk.sampleSize / 8);

	  for (i = 0; i < sizeof(char[10]); i++)
			temp_sampleRate[i] = CommChunk.sampleRate[i];

	  extended_to_double(temp_sampleRate, &aiff_ptr->sampleRate);


/* to start the search again from the beginning, HP 27-may-93 */
         fseek (file_ptr, sizeof(Chunk), SEEK_SET);

	 if (fread(&ident, sizeof(identifier), 1, file_ptr) != 1)
		return(-1);

#ifdef MSDOS

	holder = ident.name[0];
	ident.name[0] = ident.name[3];
	ident.name[3] = holder;
	holder = ident.name[1];
	ident.name[1] = ident.name[2];
	ident.name[2] = holder;

	ident.ck_length = _lrotl(ident.ck_length, 8);

#endif

/* fixed bug in next line, HP 27-may-93 */
	while(strncmp(ident.name,IFF_ID_SSND,4) != 0)
	{
		dummy = calloc( ident.ck_length, sizeof(char));
/* changed "fread( &dummy," to "fread ( dummy,", HP 26-may-93 */
		if(fread( dummy, ident.ck_length, 1, file_ptr) != 1)
		  return(-1);
		free(dummy);
		if(fread( &ident, sizeof(identifier),1, file_ptr) != 1)
		  return (-1);
/* the following lines are not necessary, HP 27-may-93 */
/*
		  {
		  fseek(file_ptr, 0, SEEK_SET);
		  if(fread( &ident, sizeof(identifier), 1, file_ptr) != 1)
		  return(-1);
		  }
*/

#ifdef MSDOS


	holder = ident.name[0];
	ident.name[0] = ident.name[3];
	ident.name[3] = holder;
	holder = ident.name[1];
	ident.name[1] = ident.name[2];
	ident.name[2] = holder;

	ident.ck_length = _lrotl(ident.ck_length, 8);

#endif
	}

		for(i = 0; i < 4; ++i)
		SndDChunk.ckID[i] = ident.name[i];

		SndDChunk.ckSize = ident.ck_length;

		if (fread(&SndDChunk.offset, sizeof(unsigned long), 1, file_ptr) != 1)
			return(-1);

		if (fread(&SndDChunk.blockSize, sizeof(unsigned long), 1,
             file_ptr) != 1)
			return(-1);

#ifdef MSDOS

  SndDChunk.offset = _lrotl(SndDChunk.offset, 8);
  SndDChunk.blockSize = _lrotl(SndDChunk.blockSize, 8);

#endif

/* why seek behinde the SSND Chunk ????, HP 27-may-93 */
/*
   seek_offset = SndDChunk.ckSize - sizeof(SoundDataChunk) +
      sizeof(ChunkHeader);

   if (fseek(file_ptr, seek_offset, SEEK_CUR) != 0)
	  return(-1);
*/

   aiff_ptr->numChannels       = CommChunk.numChannels;
   aiff_ptr->numSampleFrames   = CommChunk.numSampleFrames;
   aiff_ptr->sampleSize        = CommChunk.sampleSize;
   aiff_ptr->blkAlgn.offset    = SndDChunk.offset;
   aiff_ptr->blkAlgn.blockSize = SndDChunk.blockSize;
   strncpy(aiff_ptr->sampleType, SndDChunk.ckID, 4);

   return(0);

}


/*****************************************************************************
*
*  Seek past some Audio Interchange File Format (AIFF) headers to sound data.
*
*****************************************************************************/

int aiff_seek_to_sound_data(FILE *file_ptr)
{

   if (fseek(file_ptr, sizeof(Chunk) + sizeof(SoundDataChunk), SEEK_SET) != 0)
		return(-1);
	else

   return(0);

}

/*******************************************************************************
*
*  Write Audio Interchange File Format (AIFF) headers.
*
*******************************************************************************/

int aiff_write_headers(FILE *file_ptr,
		       IFF_AIFF *aiff_ptr)
{

register char   i;
register long   seek_offset;

char            temp_sampleRate[10];

Chunk           FormChunk;
CommonChunk     CommChunk;
SoundDataChunk  SndDChunk;


	strcpy( FormChunk.ckID, IFF_ID_FORM);
	strcpy( FormChunk.formType, IFF_ID_AIFF);
	strcpy( CommChunk.ckID, IFF_ID_COMM);  /*7/7/93,SR,changed FormChunk to CommChunk*/


   double_to_extended(&aiff_ptr->sampleRate, temp_sampleRate);

   for (i = 0; i < sizeof(char[10]); i++)
      CommChunk.sampleRate[i] = temp_sampleRate[i];

   CommChunk.numChannels             = aiff_ptr->numChannels;
   CommChunk.numSampleFrames         = aiff_ptr->numSampleFrames;
   CommChunk.sampleSize              = aiff_ptr->sampleSize;
   SndDChunk.offset                  = aiff_ptr->blkAlgn.offset;
   SndDChunk.blockSize               = aiff_ptr->blkAlgn.blockSize;
   strncpy(/*(unsigned long *)*/ SndDChunk.ckID, aiff_ptr->sampleType,  4);

   CommChunk.ckSize = sizeof(CommChunk.numChannels) +
      sizeof(CommChunk.numSampleFrames) + sizeof(CommChunk.sampleSize) +
      sizeof(CommChunk.sampleRate);

   SndDChunk.ckSize = sizeof(SoundDataChunk) - sizeof(ChunkHeader) +
      (CommChunk.sampleSize + BITS_IN_A_BYTE - 1) / BITS_IN_A_BYTE *
      CommChunk.numChannels * CommChunk.numSampleFrames;

   FormChunk.ckSize = sizeof(Chunk) + SndDChunk.ckSize + sizeof(ChunkHeader) +
	  CommChunk.ckSize;

   if (fseek(file_ptr, 0, SEEK_SET) != 0)
      return(-1);

   if (fwrite(&FormChunk, sizeof(Chunk), 1, file_ptr) != 1)
      return(-1);

   if (fwrite(&SndDChunk, sizeof(SoundDataChunk), 1, file_ptr) != 1)
      return(-1);

   seek_offset = SndDChunk.ckSize - sizeof(SoundDataChunk) +
      sizeof(ChunkHeader);

   if (fseek(file_ptr, seek_offset, SEEK_CUR) != 0)
      return(-1);

   if (fwrite(CommChunk.ckID, sizeof(ID), 1, file_ptr) != 1)
      return(-1);

   if (fwrite(&CommChunk.ckSize, sizeof(long), 1, file_ptr) != 1)
      return(-1);

   if (fwrite(&CommChunk.numChannels, sizeof(short), 1, file_ptr) != 1)
      return(-1);

   if (fwrite(&CommChunk.numSampleFrames, sizeof(unsigned long), 1,
	      file_ptr) != 1)
	  return(-1);

   if (fwrite(&CommChunk.sampleSize, sizeof(short), 1, file_ptr) != 1)
	  return(-1);

   if (fwrite(CommChunk.sampleRate, sizeof(char[10]), 1, file_ptr) != 1)
      return(-1);

   return(0);

}

/*****************************************************************************
*
*  bit_stream.c package
*  Author:  Jean-Georges Fritsch, C-Cube Microsystems
*
*****************************************************************************/

/********************************************************************
  This package provides functions to write (exclusive or read)
  information from (exclusive or to) the bit stream.

  If the bit stream is opened in read mode only the get functions are
  available. If the bit stream is opened in write mode only the put
  functions are available.
********************************************************************/

/*open_bit_stream_w(); open the device to write the bit stream into it    */
/*open_bit_stream_r(); open the device to read the bit stream from it     */
/*close_bit_stream();  close the device containing the bit stream         */
/*alloc_buffer();      open and initialize the buffer;                    */
/*desalloc_buffer();   empty and close the buffer                         */
/*back_track_buffer();     goes back N bits in the buffer                 */
/*unsigned int get1bit();  read 1 bit from the bit stream                 */
/*unsigned long getbits(); read N bits from the bit stream                */
/*unsigned long byte_ali_getbits();   read the next byte aligned N bits from*/
/*                                    the bit stream                        */
/*unsigned long look_ahead(); grep the next N bits in the bit stream without*/
/*                            changing the buffer pointer                   */
/*put1bit(); write 1 bit from the bit stream  */
/*put1bit(); write 1 bit from the bit stream  */
/*putbits(); write N bits from the bit stream */
/*byte_ali_putbits(); write byte aligned the next N bits into the bit stream*/
/*unsigned long sstell(); return the current bit stream length (in bits)    */
/*int end_bs(); return 1 if the end of bit stream reached otherwise 0       */
/*int seek_sync(); return 1 if a sync word was found in the bit stream      */
/*                 otherwise returns 0                                      */
/* refill the buffer from the input device when the buffer becomes empty    */
/****************************************************************************/
/* 7/13/1994 Ralf Schwalbe  new funktions                                   */
/* local_get1bit()  the same as get1bit: get1bit is changed for extension   */
/* local_getbits()                                                          */
/* get1bit()	    to read in extension bitstream                          */
/* getbits()        to read in extension bitstream			    */				    
/* skip_bits()								    */
/* skip_ext_bits()							    */
/****************************************************************************/


/* refill the buffer from the input device when the buffer becomes empty    */
void refill_buffer(Bit_stream_struc *bs)      /* bit stream structure */
{
   register int i=bs->buf_size-2-bs->buf_byte_idx;
   register unsigned long n;
   register int index=0;
   char val[2];

   while ((i>=0) && (!bs->eob)) {

	  if (bs->format == BINARY)
         n = fread(&bs->buf[i--], sizeof(unsigned char), 1, bs->pt);

	  else {
	  while((index < 2) && n) {
		n = fread(&val[index], sizeof(char), 1, bs->pt);
		switch (val[index]) {
			case 0x30:
			case 0x31:
			case 0x32:
			case 0x33:
			case 0x34:
			case 0x35:
			case 0x36:
			case 0x37:
			case 0x38:
			case 0x39:
			case 0x41:
			case 0x42:
			case 0x43:
			case 0x44:
			case 0x45:
			case 0x46:
			index++;
			break;
			default: break;
		}
	   }

           if (val[0] <= 0x39) bs->buf[i] = (val[0] - 0x30) << 4;
                 else  bs->buf[i] = (val[0] - 0x37) << 4;
		   if (val[1] <= 0x39) bs->buf[i--] |= (val[1] - 0x30);
				 else  bs->buf[i--] |= (val[1] - 0x37);
	   index = 0;
      }

      if (!n) {
         bs->eob= i+1;
      }

	}
}
/* 10.02.1995 R.S. */
/* refill the buffer from the input device when the ext.-buffer becomes empty */

static char *he = "0123456789ABCDEF";

/* empty the buffer to the output device when the buffer becomes full */
void empty_buffer(Bit_stream_struc *bs,       /* bit stream structure */
		  int minimum           /* end of the buffer to empty */)
{
 register int i;

#if BS_FORMAT == BINARY
   for (i=bs->buf_size-1;i>=minimum;i--)
      fwrite(&bs->buf[i], sizeof(unsigned char), 1, bs->pt);
#else
   for (i=bs->buf_size-1;i>=minimum;i--) {
       char val[2];
       val[0] = he[((bs->buf[i] >> 4) & 0x0F)];
       val[1] = he[(bs->buf[i] & 0x0F)];
	   fwrite(val, sizeof(char), 2, bs->pt);
   }
#endif

   for (i=minimum-1; i>=0; i--)
       bs->buf[bs->buf_size - minimum + i] = bs->buf[i];

   bs->buf_byte_idx = bs->buf_size -1 - minimum;
   bs->buf_bit_idx = 8;
}


/* empty the buffer to the output device when the buffer becomes full */
void empty_buffer_concat(Bit_stream_struc *bs,       /* bit stream structure */
		  int minimum           /* end of the buffer to empty */)
{
 register int i;
 int j;

#if BS_FORMAT == BINARY
	j==0;
   for (i=bs->buf_size-1;i>=minimum;i--)
   {
   if(j==7)
   	{
	fwrite("\nfltbits 0x", sizeof(unsigned char), 12, bs->pt);
	j=0;
	}   	
   fwrite(&bs->buf[i], sizeof(unsigned char), 1, bs->pt);
   j++;
   }
#else
	j=0;
	fwrite("\nfltbits 0x", sizeof(char), 11, bs->pt);
   for (i=bs->buf_size-1;i>=minimum;i--) {
       char val[2];
   if(j==4)
   	{
	fwrite("\nfltbits 0x", sizeof(char), 11, bs->pt);
	j=0;
	}   	
       val[0] = he[((bs->buf[i] >> 4) & 0x0F)];
       val[1] = he[(bs->buf[i] & 0x0F)];
	   fwrite(val, sizeof(char), 2, bs->pt);
	   j++;
   }
#endif

   for (i=minimum-1; i>=0; i--)
       bs->buf[bs->buf_size - minimum + i] = bs->buf[i];

   bs->buf_byte_idx = bs->buf_size -1 - minimum;
   bs->buf_bit_idx = 8;
}


/* open the device to write the bit stream into it */
void open_bit_stream_w(Bit_stream_struc *bs,   /* bit stream structure */
		       char *bs_filenam,       /* name of the bit stream file */
		       int size                /* size of the buffer */)
{
   if ((bs->pt = fopen(bs_filenam, "w+")) == NULL) {
      printf("Could not create \"%s\".\n", bs_filenam);
      exit(0);
   }
   alloc_buffer(bs, size);
   bs->buf_byte_idx = size-1;
   bs->buf_bit_idx=8;
   bs->totbit=0;
   bs->mode = WRITE_MODE;
   bs->eob = FALSE;
   bs->eobs = FALSE;
}

/* open the device to read the bit stream from it */
int open_bit_stream_r(Bit_stream_struc *bs,   /* bit stream structure */
		      char *bs_filenam,       /* name of the bit stream file */
		      int size                /* size of the buffer */)
{
   register unsigned long n;
   register int i=0,j=0;
   register unsigned char flag = 1;
   unsigned char val;

   if ((bs->pt = fopen(bs_filenam, "rb")) == NULL) {
	  printf("Could not find \"%s\".\n", bs_filenam);
	  return(0);
   }

   do {
	 n = fread(&val, sizeof(unsigned char), 1, bs->pt);
	 switch (val) {
	  case 0x30:
	  case 0x31:
	  case 0x32:
	  case 0x33:
	  case 0x34:
	  case 0x35:
	  case 0x36:
	  case 0x37:
	  case 0x38:
	  case 0x39:
	  case 0x41:
	  case 0x42:
	  case 0x43:
	  case 0x44:
	  case 0x45:
	  case 0x46:  j++;
	  case 0xa:  /* \n */
		  break;

	  default: /* detection of an binary character */
		  flag--;
		  i = 300;
		  break;
	 }

   } while (flag & n);

   if (flag) {
	  printf ("the bit stream file %s is an ASCII file\n", bs_filenam);
	  bs->format = ASCII;
   }
   else {
	  bs->format = BINARY;
	  printf ("the bit stream file %s is a BINARY file\n", bs_filenam);
   }

   fclose(bs->pt);

   if ((bs->pt = fopen(bs_filenam, "rb")) == NULL) {
      printf("Could not find \"%s\".\n", bs_filenam);
      return(0);
   }

   alloc_buffer(bs, size);
   bs->buf_byte_idx=0;
   bs->buf_bit_idx=0;
   bs->totbit=0;
   bs->mode = READ_MODE;
   bs->eob = FALSE;
   bs->eobs = FALSE;
   return(1);
}

/*close the device containing the bit stream after a read process*/
void close_bit_stream_r(Bit_stream_struc *bs)
{
   fclose(bs->pt);
   desalloc_buffer(bs);
}

/*close the device containing the bit stream after a write process*/
void close_bit_stream_w(Bit_stream_struc *bs)
{
   empty_buffer(bs, bs->buf_byte_idx);
   fclose(bs->pt);
   desalloc_buffer(bs);
}

void close_bit_stream_w_concat(Bit_stream_struc *bs)
{
   empty_buffer_concat(bs, bs->buf_byte_idx);
   fclose(bs->pt);
   desalloc_buffer(bs);
}

/*open and initialize the buffer; */
void alloc_buffer(Bit_stream_struc *bs, int size)
{
   bs->buf = (unsigned char /*far*/ *) mem_alloc(size*sizeof(unsigned
              char), "buffer");
   bs->buf_size = size;
}

/*empty and close the buffer */
void desalloc_buffer(Bit_stream_struc *bs)
{
   free(bs->buf);
}

int putmask[9]={0x0, 0x1, 0x3, 0x7, 0xf, 0x1f, 0x3f, 0x7f, 0xff};
int clearmask[9]={0xff, 0xfe, 0xfc, 0xf8, 0xf0, 0xe0, 0xc0, 0x80, 0x0};

void back_track_buffer(Bit_stream_struc *bs, int N) /* goes back N bits in the buffer */
{
   int tmp = N - (N/8)*8;
   register int i;

   bs->totbit -= N;
   bits_in_frame -=N;
   for (i=bs->buf_byte_idx;i< bs->buf_byte_idx+N/8-1;i++) bs->buf[i] = 0;
   bs->buf_byte_idx += N/8;
   if ( (tmp + bs->buf_bit_idx) <= 8) {
      bs->buf_bit_idx += tmp;
   }
   else {
	  bs->buf_byte_idx ++;
      bs->buf_bit_idx += (tmp - 8);
   }
   bs->buf[bs->buf_byte_idx] &= clearmask[bs->buf_bit_idx];
}



int mask[8]={0x1, 0x2, 0x4, 0x8, 0x10, 0x20, 0x40, 0x80};

/*read 1 bit from the bit stream */
unsigned int local_get1bit(Bit_stream_struc *bs)
{
   unsigned int bit;
   register int i;

   bs->totbit++;
   bits_in_frame++;

   if (!bs->buf_bit_idx)
   {
		bs->buf_bit_idx = 8;
		bs->buf_byte_idx--;
		if ((bs->buf_byte_idx < MINIMUM) || (bs->buf_byte_idx < bs->eob))
		{
			 if (bs->eob)
				bs->eobs = TRUE;
			 else
			 {
				for (i=bs->buf_byte_idx; i>=0;i--)
				  bs->buf[bs->buf_size-1-bs->buf_byte_idx+i] = bs->buf[i];
				refill_buffer(bs);
				bs->buf_byte_idx = bs->buf_size-1;
			 }
		}
   }
   bit = bs->buf[bs->buf_byte_idx]&mask[bs->buf_bit_idx-1];
   bit = bit >> (bs->buf_bit_idx-1);
   bs->buf_bit_idx--;

#ifdef CONCAT
   put1bit_concat(&bs_concat, bit);	/* 23/03/1995 JMZ Concat */
#endif CONCAT

   return(bit);
}

/*write 1 bit from the bit stream */
void put1bit(Bit_stream_struc *bs, int bit)
{
   register int i;

   bs->totbit++;

   bs->buf[bs->buf_byte_idx] |= (bit&0x1) << (bs->buf_bit_idx-1);
   bs->buf_bit_idx--;
   if (!bs->buf_bit_idx) {
	   bs->buf_bit_idx = 8;
	   bs->buf_byte_idx--;
	   if (bs->buf_byte_idx < 0)
		  empty_buffer(bs, MINIMUM);
	   bs->buf[bs->buf_byte_idx] = 0;
   }
}

void put1bit_concat(Bit_stream_struc *bs, int bit)
{
   register int i;

   bs->totbit++;

   bs->buf[bs->buf_byte_idx] |= (bit&0x1) << (bs->buf_bit_idx-1);
   bs->buf_bit_idx--;
   if (!bs->buf_bit_idx) {
	   bs->buf_bit_idx = 8;
	   bs->buf_byte_idx--;
	   if (bs->buf_byte_idx < 0)
		  empty_buffer_concat(bs, MINIMUM);
	   bs->buf[bs->buf_byte_idx] = 0;
   }
}

/*look ahead for the next N bits from the bit stream */
unsigned long look_ahead(Bit_stream_struc *bs, int N)
{
 unsigned long val=0;
 register int i;
 register int j = N;
 register int k, tmp;
 
 if (N > MAX_LENGTH)
	printf("Cannot read or write more than %d bits at a time.\n", MAX_LENGTH);

 while (j > 0) {
	if (!bs->buf_bit_idx)
	{
		bs->buf_bit_idx = 8;
		bs->buf_byte_idx--;
	}
	if(bs->buf_byte_idx == -1)	/* R.S.  refill_buffer_tmp(&bs_ext); */
	k = MIN(j, bs->buf_bit_idx);
	if(bs->buf_byte_idx <= -1)
		tmp = bs_tmp_buf[(bs->buf_byte_idx * -1)-1]&putmask[bs->buf_bit_idx];
	else
		tmp = bs->buf[bs->buf_byte_idx]&putmask[bs->buf_bit_idx];
	tmp = tmp >> (bs->buf_bit_idx-k);
	val |= tmp << (j-k);
	bs->buf_bit_idx -= k;
	j -= k;
 }
 return(val);
}

/*read N bit from the bit stream */
unsigned int local_getbits(Bit_stream_struc *bs, int N)
{
	unsigned long val=0;
	register int i,r;
	register int j = N;
	register int k, tmp;

	if (N > MAX_LENGTH)
		printf("Cannot read or write more than %d bits at a time.\n", MAX_LENGTH);

	bs->totbit += N;
	bits_in_frame +=N;

	while (j > 0)
	{
		if (!bs->buf_bit_idx)
		{
			bs->buf_bit_idx = 8;
			bs->buf_byte_idx--;
			if ((bs->buf_byte_idx < MINIMUM) || (bs->buf_byte_idx < bs->eob))
			{
			 if (bs->eob)
				bs->eobs = TRUE;
			 else
			 {
				for (i=bs->buf_byte_idx; i>=0;i--)
				   bs->buf[bs->buf_size-1-bs->buf_byte_idx+i] = bs->buf[i];
				refill_buffer(bs);
				bs->buf_byte_idx = bs->buf_size-1;
			 }
			}
		}
		k = MIN (j, bs->buf_bit_idx);
		tmp = bs->buf[bs->buf_byte_idx]&putmask[bs->buf_bit_idx];
		tmp = tmp >> (bs->buf_bit_idx-k);
		val |= tmp << (j-k);
		bs->buf_bit_idx -= k;
		j -= k;
	}

#ifdef CONCAT
 putbits_concat(&bs_concat, val, N); /*23/03/1995 JMZ Concat */
#endif CONCAT

 return(val);
}
/************************************************************************/
/* some new funktions to read bits in mc - and extension part 		*/
/* 30.06.1994 Ralf Schwalbe                                    		*/
/************************************************************************/

void skipbits(Bit_stream_struc *bs, unsigned int n)
{
	while( n >= MAX_LENGTH )
	{
		info.n_ad_bytes -= 4;
		getbits(bs, MAX_LENGTH);
		n -= MAX_LENGTH;
	}
	if( n > 0 )
	{
		info.n_ad_bytes --;
		getbits(bs, n);
	}
}

void skip_ext_bits(Bit_stream_struc *bs, unsigned int n)
{
	while( n >= MAX_LENGTH )
	{
		local_getbits(&bs_ext, MAX_LENGTH);
		n -= MAX_LENGTH;
	}
	if( n > 0 )
		local_getbits(&bs_ext, n);

}

unsigned int get1bit(Bit_stream_struc *bs)
{
	if (read_from_ext)
	{
		mc_ext_header();
		if(ext_error) return(0);   /* R.S. mute ext. samples */
		else return(local_get1bit(&bs_ext));
	}
	else
	{
		if( bits_in_frame == (9216 - (info.n_ad_bytes*8)) )
		{
			read_from_ext = 1;
			skipbits(bs, info.n_ad_bytes*8);
		}
		return(local_get1bit(bs));
	}
}

unsigned long getbits(Bit_stream_struc *bs, int n)
{
	unsigned long val = 0;
	int  bs_cnt,bs_ext_cnt;

	if( read_from_ext )
	{
		mc_ext_header();
		if(ext_error) return(0); /* R.S. mute ext. samples */
		else return(local_getbits(&bs_ext, n));
	}
	else
	{
		if( info.ext_bit_stream_present )
		{
			if( bits_in_frame <= ((9216 - info.n_ad_bytes*8) - n) )
			{
				if ( bits_in_frame == 9216 )
				{
					read_from_ext = 1;
					skipbits(bs, info.n_ad_bytes*8);
				}
				return(local_getbits(bs, n));
			}
			else
			{
				bs_cnt = 9216 - (info.n_ad_bytes*8) - bits_in_frame;
				bs_ext_cnt = n - bs_cnt;
				val = (local_getbits(bs, bs_cnt)) << bs_ext_cnt;
				skipbits(bs, info.n_ad_bytes*8);
				read_from_ext = 1;
				if(mc_ext_header()==1) 
					val |= getbits(&bs_ext, bs_ext_cnt);
				else	val = 0; /* R.S. else error in ext_crc */
				return(val);
			}
		}
		else    return(local_getbits(bs, n));
	}
}

void program_information(void)
{
 char version[5]; 
 printf("\n MPEG 2 multichannel decoding program ");
 printf("\n--------------------------------------");
 strcpy(version,VERSION);
 printf("\n Shareware Version: %s , MPEG 2 IS 13818-3\n\n",VERSION);
} 

/*******************************************************************/


/*write N bits into the bit stream */
void putbits(Bit_stream_struc *bs,   /* bit stream structure */
	     unsigned int val,       /* val to write into the buffer */
	     int N                   /* number of bits of val */)
{
 register int i;
 register int j = N;
 register int k, tmp;

 if (N > MAX_LENGTH)
	printf("Cannot read or write more than %d bits at a time.\n", MAX_LENGTH);

 bs->totbit += N;
 while (j > 0) {
   k = MIN(j, bs->buf_bit_idx);
   tmp = val >> (j-k);
   bs->buf[bs->buf_byte_idx] |= (tmp&putmask[k]) << (bs->buf_bit_idx-k);
   bs->buf_bit_idx -= k;
   if (!bs->buf_bit_idx) {
	   bs->buf_bit_idx = 8;
       bs->buf_byte_idx--;
	   if (bs->buf_byte_idx < 0)
          empty_buffer(bs, MINIMUM);
       bs->buf[bs->buf_byte_idx] = 0;
   }
   j -= k;
 }
}

/*write N bits into the bit stream */
void putbits_concat(Bit_stream_struc *bs,   /* bit stream structure */
	     unsigned int val,       /* val to write into the buffer */
	     int N                   /* number of bits of val */)
{
 register int i;
 register int j = N;
 register int k, tmp;

 if (N > MAX_LENGTH)
	printf("Cannot read or write more than %d bits at a time.\n", MAX_LENGTH);

 bs->totbit += N;
 while (j > 0) {
   k = MIN(j, bs->buf_bit_idx);
   tmp = val >> (j-k);
   bs->buf[bs->buf_byte_idx] |= (tmp&putmask[k]) << (bs->buf_bit_idx-k);
   bs->buf_bit_idx -= k;
   if (!bs->buf_bit_idx) {
	   bs->buf_bit_idx = 8;
       bs->buf_byte_idx--;
	   if (bs->buf_byte_idx < 0)
          empty_buffer_concat(bs, MINIMUM);
       bs->buf[bs->buf_byte_idx] = 0;
   }
   j -= k;
 }
}

/*write N bits byte aligned into the bit stream */
void byte_ali_putbits(Bit_stream_struc *bs,   /* bit stream structure */
		      unsigned int val,       /* val to write into the buffer */
		      int N                   /* number of bits of val */)
{
  unsigned long aligning, sstell(Bit_stream_struc *bs);
  /* register int tmp =0; */ /*21.9.93 R.S. not used */

  if (N > MAX_LENGTH)
	printf("Cannot read or write more than %d bits at a time.\n", MAX_LENGTH);
  aligning = sstell(bs)%8;
  if (aligning)

#ifdef  MS_DOS
  putbits(bs, (unsigned int)0, (int)(8-aligning)); /* casting, 92-07-08 sr */
#else
  putbits(bs, 0, 8-aligning);
#endif

  putbits(bs, val, N);
}

/*read the next bute aligned N bits from the bit stream */
unsigned long byte_ali_getbits(Bit_stream_struc *bs, int N)
{
 unsigned long aligning, sstell(Bit_stream_struc *bs);
 unsigned long val;

 if (N > MAX_LENGTH)
	printf("Cannot read or write more than %d bits at a time.\n", MAX_LENGTH);
 aligning = sstell(bs)%8;
 if (aligning)
  	getbits(bs, 8-aligning);

 return(getbits(bs, N));
}

/*return the current bit stream length (in bits)*/
unsigned long sstell(Bit_stream_struc *bs)
{
  return(bs->totbit);
}

/*return the status of the bit stream*/
/* returns 1 if end of bit stream was reached */
/* returns 0 if end of bit stream was not reached */
int end_bs(Bit_stream_struc *bs)
{
  return(bs->eobs);
}

/*this function seeks for a byte aligned sync word in the bit stream and
  places the bit stream pointer right after the sync.
  This function returns 1 if the sync was found otherwise it returns 0 */

int seek_sync(Bit_stream_struc *bs,   /* bit stream structure */
	      long sync,              /* sync word maximum 32 bits */
	      int N        	      /* sync word length */)
{
 double pow(double, double);
 unsigned long aligning, stell();
 unsigned long val;
 long maxi = (int)pow(2.0, (FLOAT)N) - 1;

 if( (bits_in_frame < 9216) && (frameNum != 0))
	skipbits(bs, (9216 - bits_in_frame));

 if(read_ext_header)
	if(bits_in_frame < (9216 + ( info.ext_length * 8)))
	    skipbits(&bs_ext, ((9216 + ( info.ext_length * 8))
				   - bits_in_frame));

 bits_in_frame = 0;
 read_from_ext = 0;
 read_ext_header = 0;
 ext_error = 0;
 aligning = sstell(bs)%ALIGNING;
 if (aligning)
	getbits(bs, (int) (ALIGNING-aligning)); /*sr*/

 val = getbits(bs, N);
 while (((val&maxi) != sync) && (!end_bs(bs)))
 {
	val <<= ALIGNING;
	val |= getbits(bs, ALIGNING);
 }

 if (end_bs(bs)) return(0);
 else return(1);

}

/*****************************************************************************
*
*  End of bit_stream.c package
*
*****************************************************************************/

/*****************************************************************************
*
*  CRC error protection package
*
*****************************************************************************/

void II_CRC_calc(frame_params *fr_ps,
		 unsigned int bit_alloc[5][SBLIMIT], 
		 unsigned int scfsi[5][SBLIMIT],
		 unsigned int *crc)
{
        int i, k;
	layer *info = fr_ps->header;
        int stereo  = fr_ps->stereo;
        int sblimit = fr_ps->sblimit;
        int jsbound = fr_ps->jsbound;
	al_table *alloc = fr_ps->alloc;

        *crc = 0xffff; /* changed from '0' 92-08-11 shn */
        update_CRC(info->bitrate_index, 4, crc);
        update_CRC(info->sampling_frequency, 2, crc);
        update_CRC(info->padding, 1, crc);
        update_CRC(info->extension, 1, crc);
        update_CRC(info->mode, 2, crc);
        update_CRC(info->mode_ext, 2, crc);
	update_CRC(info->copyright, 1, crc);
	update_CRC(info->original, 1, crc);
	update_CRC(info->emphasis, 2, crc);

	for (i=0;i<sblimit;i++)
		for (k=0;k<((i<jsbound)?stereo:1);k++)
			update_CRC(bit_alloc[k][i], (*alloc)[i][0].bits, crc);

	for (i=0;i<sblimit;i++)
		for (k=0;k<stereo;k++)
			if (bit_alloc[k][i])
				update_CRC(scfsi[k][i], 2, crc);
}


void mc_error_check(frame_params *fr_ps,
		    unsigned int bit_alloc[7][SBLIMIT], 
		    unsigned int scfsi[7][SBLIMIT],
		    unsigned int *crc)
{
	 int i, m, l, pci;
	 layer *info = fr_ps->header;
	 int sblimit = fr_ps->sblimit;
	 int jsbound = fr_ps->jsbound; /* R.S. 30.5. */
	 al_table *alloc = fr_ps->alloc;

	 *crc = 0xffff; /* changed from '0' 92-08-11 shn */
	 update_CRC(info->ext_bit_stream_present, 1, crc);
	 if ( info->ext_bit_stream_present == 1)
		update_CRC(info->ad_bytes_crc_info, 8, crc);
	 update_CRC(info->center, 2, crc);
	 update_CRC(info->surround, 2, crc);
	 update_CRC(info->lfe, 1, crc);
	 update_CRC(info->audio_mix, 1,crc);
	 update_CRC(info->dematrix_procedure, 2, crc);
	 update_CRC(info->no_of_multi_lingual_ch, 3, crc);
	 update_CRC(info->multi_lingual_fs, 1, crc);
	 update_CRC(info->multi_lingual_layer, 1, crc);
	 update_CRC(info->copyright_ident_bit, 1, crc);
	 update_CRC(info->copyright_ident_start, 1, crc);
	 
	 update_CRC(info->tc_sbgr_select, 1, crc);
	 update_CRC(info->dyn_cross_on, 1, crc);
	 update_CRC(info->mc_prediction_on, 1, crc);

	 if(info->tc_sbgr_select == 1)
		update_CRC(info->tc_allocation, 3, crc);
	 else
		for(i = 0; i < 12; i++)
			update_CRC(info->tc_alloc[i], 3, crc);

	 if(info->dyn_cross_on == 1)
	 {
		update_CRC(info->dyn_cross_LR, 1, crc);
		for(i = 0; i < 12; i++)
			update_CRC(info->dyn_cross_mode[i], 4,crc);
	 }

	 /*if( info->mc_prediction_on == 1)*/
	
	 for (i=0;i<sblimit;i++)
		if( i < jsbound)
			for(m = 2; m < 5; ++m)
			{
				if((i < 12) || (m != 2) || (fr_ps ->header->center != 3))
					update_CRC(bit_alloc[m][i], (*alloc)[i][0].bits, crc);
					/* to limit the CRC calc., if the phant. center valid */
			}


	 for (i=0;i<sblimit;i++)
		for(m = 2; m < 5; ++m)
			if (bit_alloc[m][i])
				update_CRC(scfsi[m][i], 2, crc);

}

int mc_ext_error_check(void)
{
	 int i;
	 unsigned int crc;

	 crc = 0xffff;
	 update_CRC(info.ext_length, 11, &crc);
	 update_CRC(info.reserved_bit, 1, &crc);
	 for( i = 0; i < 14; i++)
		update_CRC(info.ext_crc_bits[i], 8, &crc);
	 update_CRC(info.ext_crc_bits[i], 4, &crc);

	 if (crc != info.ext_crc_check)
	 {
		printf("\nERROR in EXT.-CRC \n");
		ext_error=1;
		return(0);
	 }
	 else
	 return(1);
}

void update_CRC(unsigned int data,
		unsigned int length,
		unsigned int *crc)
{
		unsigned int  masking, carry;

		masking = 1 << length;

		while((masking >>= 1)){
				carry = *crc & 0x8000;
				*crc <<= 1;
				if (!carry ^ !(data & masking))
						*crc ^= CRC16_POLYNOMIAL;
		}
		*crc &= 0xffff;
}

/*****************************************************************************
*
*  End of CRC error protection package
*
*****************************************************************************/

#ifdef  MACINTOSH
/*****************************************************************************
*
*  Set Macintosh file attributes.
*
*****************************************************************************/

void    set_mac_file_attr(char fileName[MAX_NAME_SIZE],
			  short vRefNum,
			  OsType creator,
			  OsType fileType)
{

short   theFile;
char    pascal_fileName[MAX_NAME_SIZE];
FInfo   fndrInfo;

		CtoPstr(strcpy(pascal_fileName, fileName));

        FSOpen(pascal_fileName, vRefNum, &theFile);
        GetFInfo(pascal_fileName, vRefNum, &fndrInfo);
		fndrInfo.fdCreator = creator;
        fndrInfo.fdType = fileType;
        SetFInfo(pascal_fileName, vRefNum, &fndrInfo);
        FSClose(theFile);

}
#endif

#ifdef  MSDOS
/* ------------------------------------------------------------------------
new_ext.
Puts a new extension name on a file name <filename>.
Removes the last extension name, if any.
92-08-19 shn
------------------------------------------------------------------------ */
char *new_ext(char *filename, char *extname)
{
  int found, dotpos;
  char newname[80];

  /* First, strip the extension */
  dotpos=strlen(filename); found=0;
  do
  {
    switch (filename[dotpos])
	{
      case '.' : found=1; break;
      case '\\': 
      case '/' :
      case ':' : found=-1; break;
      default  : dotpos--; if (dotpos<0) found=-1; break;
    }
  } while (found==0);
  if (found==-1) strcpy(newname,filename);
  if (found== 1) strncpy(newname,filename,dotpos); newname[dotpos]='\0';
  strcat(newname,extname);
  return(newname);
}
#endif

