/**********************************************************************
 * Copyright (c) 1995 MPEG/audio software simulation group
 * All Rights Reserved
 * $Id: common.h,v 1.4 1995/04/06 05:49:00 rowlands Exp $
 *
 * MPEG/audio coding/decoding software, work in progress
 *   NOT for public distribution until verified and approved by the
 *   MPEG/audio committee.
 *
 * $Log: common.h,v $
 * Revision 1.4  1995/04/06  05:49:00  rowlands
 * Moved machine #define to makefile
 *
 * Revision 1.3  1995/04/06  04:29:39  rowlands
 * Added header and RCS info to initial revision.
 *
 **********************************************************************/

/**********************************************************************
 * VERSION 2.5                                                        *
 *   changes made since last update:                                  *
 *   date   programmers         comment                               *
 * 2/25/91  Doulas Wong,        start of version 1.0 records          *
 *          Davis Pan                                                 *
 * 5/10/91  W. Joseph Carter    Reorganized & renamed all ".h" files  *
 *                              into "common.h" and "encoder.h".      *
 *                              Ported to Macintosh and Unix.         *
 *                              Added additional type definitions for *
 *                              AIFF, double/SANE and "bitstream.c".  *
 *                              Added function prototypes for more    *
 *                              rigorous type checking.               *
 * 27jun91  dpwe (Aware)        Added "alloc_*" defs & prototypes     *
 *                              Defined new struct 'frame_params'.    *
 *                              Changed info.stereo to info.mode_ext  *
 *                              #define constants for mode types      *
 *                              Prototype arguments if PROTO_ARGS     *
 * 5/28/91  Earle Jennings      added MS_DOS definition               *
 *                              MsDos function prototype declarations *
 * 7/10/91  Earle Jennings      added FLOAT definition as double      *
 *10/ 3/91  Don H. Lee          implemented CRC-16 error protection   *
 * 2/11/92  W. Joseph Carter    Ported new code to Macintosh.  Most   *
 *                              important fixes involved changing     *
 *                              16-bit ints to long or unsigned in    *
 *                              bit alloc routines for quant of 65535 *
 *                              and passing proper function args.     *
 *                              Removed "Other Joint Stereo" option   *
 *                              and made bitrate be total channel     *
 *                              bitrate, irrespective of the mode.    *
 *                              Fixed many small bugs & reorganized.  *
 *                              Modified some function prototypes.    *
 *                              Changed BUFFER_SIZE back to 4096.     *
 * 8 jul 92 Susanne Ritscher    MS-DOS, MSC 6.0 port fixes.           *
 *19 aug 92 Soren H. Nielsen    Printout of bit allocation.           *
 *                              UNIX port fixes. MS-DOS file name     *
 * 				extensions fix                        *
 * dec 92 Susanne Ritscher	Changed to multi-channel with several *
 * 				options	         		      *
 **********************************************************************
 *                                                                    *
 *                                                                    *
 *  MPEG/audio Phase 2 coding/decoding multichannel                   *
 *                                                                    *
 *  7/27/93        Susanne Ritscher,  IRT Munich                      *
 *  8/27/93        Susanne Ritscher, IRT Munich                       *
 *                 Channel-Switching is working                       *
 *  9/1/93         Susanne Ritscher,  IRT Munich                      *
 *                 all channels normalized                            *
 *  9/20/93        channel-switching is only performed at a           *
 *                 certain limit of TC_ALLOC dB, which is included    *
 *                 in encoder.h                                       *
 *                                                                    *
 *  Version 1.0 Shareware                                             *
 *                                                                    *
 *  07/12/94       Susanne Ritscher,  IRT Munich                      *
 *                 Tel: +49 89 32399 458                              *
 *                 Fax: +49 89 32399 415                              *
 *                                                                    *
 *  Version 1.1 Shareware                                             *
 *                                                                    *
 *  02/23/95	   Susanne Ritscher,  IRT Munich                      *
 *                 corrected some bugs                                *
 *                 extension bitstream is working                     *
 *                                                                    *
 **********************************************************************/

/***********************************************************************
*
*  Global Conditional Compile Switches
*
***********************************************************************/
/*#define PRINTOUT*/
/*#define	UNIX */		/* Unix conditional compile switch */
/*#define	MACINTOSH */	/* Macintosh conditional compile switch */
/*#define	MS_DOS	*/	/* IBM PC conditional compile switch
				   Microsoft C ver. 6.0 */


#ifdef  UNIX
#define         TABLES_PATH     "tables/"  /* to find data files */
/* name of environment variable holding path of table files */
#define         MPEGTABENV      "MPEGTABLES"
#define         PATH_SEPARATOR  "/"        /* how to build paths */
/* #define      PROTO_ARGS                 /* unix gcc uses arg. prototypes */
#endif  /* UNIX */

#ifdef  MACINTOSH
/* #define      TABLES_PATH ":tables:"  /* where to find data files */
#define         PROTO_ARGS              /* Mac uses argument prototypes */
#endif  /* MACINTOSH */

#ifdef  MS_DOS
#define         PROTO_ARGS  /* DOS uses argument prototypes */
#endif  /* MS_DOS */

/* MS_DOS and VMS do not define TABLES_PATH, so OpenTableFile will default
   to finding the data files in the default directory */

/***********************************************************************
*
*  Global Include Files
*
***********************************************************************/

#include        <stdio.h>
#include        <string.h>
#include        <math.h>

#ifdef  UNIX
#include        <unistd.h>	 /* removed 92-08-05 shn */
#include	<stdlib.h>       /* put in 92-08-05 shn */
#endif  /* UNIX */

#ifdef  MACINTOSH
#include        <stdlib.h>
#include        <console.h>
#endif  /* MACINTOSH */

#ifdef  MS_DOS
/* #include	<alloc.h> */ /* removed 92-07-08 sr */
#include	<malloc.h>   /* put in 92-07-08 sr */
/* #include	<mem.h> */   /* removed 92-07-08 sr */
#include <stdlib.h>
#endif  /* MS_DOS */

/***********************************************************************
*
*  Global Definitions
*
***********************************************************************/

/* General Definitions */

#ifdef  MS_DOS
#define         FLOAT                   double
#else
#define         FLOAT                   float
#endif

#define         FALSE                   0
#define         TRUE                    (!FALSE)
#define         NULL_CHAR               '\0'

#define         MAX_U_32_NUM            0xFFFFFFFF
#define         PI                      3.14159265358979
#define         PI4                     PI/4
#define         PI64                    PI/64
#define         LN_TO_LOG10             0.2302585093

#define         VOL_REF_NUM             0
#define         MPEG_AUDIO_ID           1
#define         MAC_WINDOW_SIZE         24

#define         MONO                    1
#define         STEREO                  2
#define         BITS_IN_A_BYTE          8
#define         WORD                    16
#define         MAX_NAME_SIZE           81
#define         SBLIMIT                 32
#define         FFT_SIZE                1024
#define         HAN_SIZE                512
#define         SCALE_BLOCK             12
#define         SCALE_RANGE             64
#define         SCALE                   32768
#define         CRC16_POLYNOMIAL        0x8005

#define         MAX_PRED_COEFF          3.96875     /* gleichfoermige Quantisierung der */
                                                    /* Praediktorkoeff. im Bereich */
                                                    /* [-MAX_PRED_COEF .. +MAX_PRED_COEF] */


/* MPEG Header Definitions - Mode Values */

#define         MPG_MD_STEREO           0
#define         MPG_MD_JOINT_STEREO     1
#define         MPG_MD_DUAL_CHANNEL     2
#define		MPG_MD_MONO		3
#define		MPG_MD_NONE		4

/* Multi-channel Definitions - Mode Values */

#define		MPG_MC_STEREO 		0

/* AIFF Definitions */

#ifndef	MS_DOS
#define         IFF_ID_FORM             "FORM" /* HP400 unix v8.0: double qoutes 1992-07-24 shn */
#define         IFF_ID_AIFF             "AIFF"
#define         IFF_ID_COMM             "COMM"
#define         IFF_ID_SSND             "SSND"
#define         IFF_ID_MPEG             "MPEG"
#else
#define         IFF_ID_FORM             "FORM"
#define         IFF_ID_AIFF             "AIFF"
#define         IFF_ID_COMM             "COMM"
#define         IFF_ID_SSND             "SSND"
#define         IFF_ID_MPEG             "MPEG"
#endif

/* "bit_stream.h" Definitions */

#define         MINIMUM         4    /* Minimum size of the buffer in bytes */
#define         MAX_LENGTH      32   /* Maximum length of word written or
                                        read from bit stream */
#define         READ_MODE       0
#define         WRITE_MODE      1
#define         ALIGNING        8
#define         BINARY          0
#define         ASCII           1
#define         BS_FORMAT       BINARY /* BINARY or ASCII = 2x bytes */
#define         BUFFER_SIZE     4096

#define         MIN(A, B)       ((A) < (B) ? (A) : (B))
#define         MAX(A, B)       ((A) > (B) ? (A) : (B))

/***********************************************************************
*
*  Global Type Definitions
*
***********************************************************************/

/* Structure for Reading Layer II Allocation Tables from File */

typedef struct {
        unsigned int    steps;
        unsigned int    bits;
        unsigned int    group;
        unsigned int    quant;
} sb_alloc, *alloc_ptr;

typedef sb_alloc        al_table[SBLIMIT][16];

/* Header Information Structure */

typedef struct {
    int version;
    int lay;
    int error_protection;
    int bitrate_index;
    int sampling_frequency;
    int padding;
    int extension;
    int mode;
    int mode_ext;
    int copyright;
    int original;
    int emphasis;
    int center;			 /* center present */
    int surround; 		 /* surrounds present*/
    int audio_mix;		 /* large listening room or not? */
    int matrix;			 /* type of dematricing */
    int lfe;			 /* low freequency effect channel*/
    int multiling_ch;		 /* number of multilingual channels */
    int multiling_fs;		 /* fs of main channels and ML are the same*/
    int multiling_lay;		 /* Layer IIML or Layer II ML is used*/
    int ext_bit_stream_present;  /* extension bitstream present? */
    int n_ad_bytes;              /* if yes, how many bits anc.data for mpeg2 */
    int mc_pred[8];		 /* prediction bit*/
    int tc_alloc[12];		 /* transmission channel allocation in each sbgroup*/
    int predsi[8][6];		 /* predictor select information*/
    int delay_comp[8][6];
    int mc_prediction_on;
    int tc_sbgr_select;
    int tc_allocation;
    int dyn_cross_on;
    int dyn_cross_LR;            /* center out of Lo or Ro*/
    int dyn_cross[12];
    int ext_length;
    int ext_sync;
    int ext_bit;
    int copy_ident_bit;
    int copy_ident_start;
    int dyn_second_stereo[12];
} layer, *the_layer;



/* Parent Structure Interpreting some Frame Parameters in Header */

typedef struct {
         layer                   *header;        /* raw header information */
	 int			 actual_mode;	  /* when writing IS, may forget if 0 chs */
	 int			 actual_modemc;
         al_table                *alloc;         /* bit allocation table read in */
         int                     tab_num;        /* number of table as loaded */
	 int			 stereo; 		  /* 1 for mono, 2 for stereo */
	 int			 stereomc;
	 int			 jsbound;		  /* first band of joint stereo coding */
	 int			 jsboundmc;
	 double		         mnr_min;		  /* mnr for dynamic bitallocation */
         int                     sblimit;        /* total number of sub bands */
} frame_params;

/* Double and SANE Floating Point Type Definitions */

typedef struct  IEEE_DBL_struct {
                unsigned long   hi;
                unsigned long   lo;
} IEEE_DBL;

typedef struct  SANE_EXT_struct {
                unsigned long   l1;
                unsigned long   l2;
                unsigned short  s1;
} SANE_EXT;

/* AIFF Type Definitions */

typedef char	 ID[4];

typedef struct  identifier_struct{
					 ID name;
					 long ck_length;
}identifier;


typedef struct  ChunkHeader_struct {
                ID      ckID;
                long    ckSize;
} ChunkHeader;

typedef struct  Chunk_struct {
                ID      ckID;
                long    ckSize;
                ID      formType;
} Chunk;

typedef struct  CommonChunk_struct {
                ID              ckID;
                long            ckSize;
                short           numChannels;
                unsigned long   numSampleFrames;
                short           sampleSize;
                char            sampleRate[10];
} CommonChunk;

typedef struct  SoundDataChunk_struct {
                ID              ckID;
                long            ckSize;
                unsigned long   offset;
                unsigned long   blockSize;
} SoundDataChunk;

typedef struct  blockAlign_struct {
                unsigned long   offset;
                unsigned long   blockSize;
} blockAlign;

typedef struct  IFF_AIFF_struct {
                short           numChannels;
                unsigned long   numSampleFrames;
                short           sampleSize;
                double          sampleRate;
                ID/*char**/     sampleType;/*must be allocated 21.6.93 SR*/
                blockAlign      blkAlgn;
} IFF_AIFF;

/* "bit_stream.h" Type Definitions */

typedef struct  bit_stream_struc {
    FILE        *pt;            /* pointer to bit stream device */
    unsigned char *buf;         /* bit stream buffer */
    int         buf_size;       /* size of buffer (in number of bytes) */
    long        totbit;         /* bit counter of bit stream */
    int         buf_byte_idx;   /* pointer to top byte in buffer */
    int         buf_bit_idx;    /* pointer to top bit of top byte in buffer */
    int         mode;           /* bit stream open in read or write mode */
    int         eob;            /* end of buffer index */
    int         eobs;           /* end of bit stream flag */
    char        format;
/* format of file in rd mode (BINARY/ASCII) */
    } Bit_stream_struc;

/* JMZ 08/03/1995 begin SORT*/
typedef struct {
         int    tc_channel;
         int    audio_channel;
         int    subband;
         int    sbgroup;
         int    used;
         double mnr;
         int    next;
} channel_sort;
/* JMZ 08/03/1995 end SORT*/

/***********************************************************************
*
*  Global Variable External Declarations
*
***********************************************************************/

extern char     *mode_names[4];
extern char     *layer_names[3];
extern double   s_freq[4];
extern int	bitrate[3][16];
extern double	multiple[64];
extern int      sb_groups[12];
extern int      transmission_channel[8][5];
extern int      pred_coef[8][4][4];
extern int      n_pred_coef[16];          
extern int	dyn_bbal[16][12];
extern int	dyn_ch[15][3];
extern int	no_channel[8][2];

/***********************************************************************
*
*  Global Function Prototype Declarations
*
***********************************************************************/

/* The following functions are in the file "common.c" */

#ifdef  MACINTOSH
extern void           set_mac_file_attr(char[MAX_NAME_SIZE], short, OsType,
                        OsType);
#endif

#ifdef  MSDOS
extern char           *new_ext(char *filename, char *extname); /* 92-08-19 shn */
#endif

extern FILE           *OpenTableFile(char *name);
extern int            read_bit_alloc(int table, al_table (*alloc));
extern int            pick_table(frame_params *fr_ps);
extern int            js_bound(int lay, int m_ext);
extern void           hdr_to_frps(frame_params *fr_ps);
extern void           WriteHdr(frame_params *fr_ps, FILE *s);
extern void           WriteBitAlloc(unsigned int (*bit_alloc)[32], frame_params *f_p, FILE *s);
extern void           WriteScale(unsigned int (*bit_alloc)[32], unsigned int (*scfsi)[32], unsigned int (*scalar)[3][32], frame_params *fr_ps, FILE *s);
extern void           WriteSamples(int ch, unsigned int *sample, unsigned int *bit_alloc, frame_params *fr_ps, FILE *s);
extern int            NumericQ(char *s);
extern int            BitrateIndex(int layr, int bRate);
extern int            SmpFrqIndex(long int sRate);
extern int            memcheck(char *array, int test, int num);
extern void           *mem_alloc(long unsigned int block, char *item);
extern void           mem_free(void **ptr_addr);
extern void           double_to_extended(double *pd, char *ps);
extern void           extended_to_double(char *ps, double *pd);
extern int            aiff_read_headers(FILE *file_ptr, IFF_AIFF *aiff_ptr, int *byte_per_sample);
extern int            aiff_seek_to_sound_data(FILE *file_ptr);
extern int            aiff_write_headers(FILE *file_ptr, IFF_AIFF *aiff_ptr);
extern int            refill_buffer(Bit_stream_struc *bs);
extern void           empty_buffer(Bit_stream_struc *bs, int minimum);
extern void           open_bit_stream_w(Bit_stream_struc *bs, char *bs_filenam, int size);
extern void           open_bit_stream_r(Bit_stream_struc *bs, char *bs_filenam, int size);
extern void           close_bit_stream_r(Bit_stream_struc *bs);
extern void           close_bit_stream_w(Bit_stream_struc *bs);
extern void           alloc_buffer(Bit_stream_struc *bs, int size);
extern void           desalloc_buffer(Bit_stream_struc *bs);
extern void           back_track_buffer(Bit_stream_struc *bs, int N);
extern unsigned int   get1bit(Bit_stream_struc *bs);
extern void           put1bit(Bit_stream_struc *bs, int bit);
extern unsigned long  look_ahead(Bit_stream_struc *bs, int N);
extern unsigned long  getbits(Bit_stream_struc *bs, int N);
extern void           putbits(Bit_stream_struc *bs, unsigned int val, int N);
extern void           byte_ali_putbits(Bit_stream_struc *bs, unsigned int val, int N);
extern unsigned long  byte_ali_getbits(Bit_stream_struc *bs, int N);
extern unsigned long  sstell(Bit_stream_struc *bs);
extern int            end_bs(Bit_stream_struc *bs);
extern int            seek_sync(Bit_stream_struc *bs, long int sync, int N);
extern void           I_CRC_calc(frame_params *fr_ps, unsigned int (*bit_alloc)[32], unsigned int *crc);
extern void           II_CRC_calc(frame_params *fr_ps, unsigned int (*bit_alloc)[32], unsigned int (*scfsi)[32], unsigned int *crc);
extern void           update_CRC(unsigned int data, unsigned int length, unsigned int *crc);
extern void           read_absthr(float *absthr, long int table);
